#include "PreCompiledHeaders.hh"
#include "Lib/HostOsInfo.hh"

#if !defined(QT_NO_OPENGL) && defined(QT_GUI_LIB)
#define VIVY_OPENGL 1
#include <QOpenGLContext>
#else
#define VIVY_OPENGL 0
#endif

#if VIVY_WIN32
#include <qt_windows.h>
#endif

using namespace Vivy::Utils;

Qt::CaseSensitivity HostOsInfo::m_overrideFileNameCaseSensitivity = Qt::CaseSensitive;
bool HostOsInfo::m_useOverrideFileNameCaseSensitivity             = false;

#if VIVY_WIN32
static WORD
hostProcessorArchitecture()
{
    SYSTEM_INFO info;
    GetNativeSystemInfo(&info);
    return info.wProcessorArchitecture;
}
#endif

HostOsInfo::HostArchitecture
HostOsInfo::hostArchitecture()
{
#if VIVY_WIN32
    static const WORD processorArchitecture = hostProcessorArchitecture();
    switch (processorArchitecture) {
    case PROCESSOR_ARCHITECTURE_AMD64: return HostOsInfo::HostArchitectureAMD64;
    case PROCESSOR_ARCHITECTURE_INTEL: return HostOsInfo::HostArchitectureX86;
    case PROCESSOR_ARCHITECTURE_IA64: return HostOsInfo::HostArchitectureItanium;
    case PROCESSOR_ARCHITECTURE_ARM: return HostOsInfo::HostArchitectureArm;
    default: return HostOsInfo::HostArchitectureUnknown;
    }
#else
    return HostOsInfo::HostArchitectureUnknown;
#endif
}

void
HostOsInfo::setOverrideFileNameCaseSensitivity(Qt::CaseSensitivity sensitivity)
{
    m_useOverrideFileNameCaseSensitivity = true;
    m_overrideFileNameCaseSensitivity    = sensitivity;
}

void
HostOsInfo::unsetOverrideFileNameCaseSensitivity()
{
    m_useOverrideFileNameCaseSensitivity = false;
}

bool
HostOsInfo::canCreateOpenGLContext([[maybe_unused]] QString *errorMessage)
{
#if VIVY_OPENGL
    static const bool canCreate = QOpenGLContext().create();
    if (!canCreate)
        *errorMessage =
            QCoreApplication::translate("Utils::HostOsInfo", "Cannot create OpenGL context.");
    return canCreate;
#else
    return false;
#endif
}
