#pragma once

#include "PreCompiledHeaders.hh"
#include "Lib/Utils.hh"

#ifdef Q_OS_WIN
#define VIVY_HOST_EXE_SUFFIX VIVY_WIN_EXE_SUFFIX
#define VIVY_WIN32           1
#else
#define VIVY_HOST_EXE_SUFFIX ""
#define VIVY_WIN32           0
#endif // Q_OS_WIN

// Detect MacOS
#if defined(Q_OS_DARWIN) || defined(Q_OS_MACOS)
#define VIVY_MACOS 1
#else
#define VIVY_MACOS 0
#endif

// Detect Linux or UNIX
#if defined(Q_OS_LINUX)
#define VIVY_LINUX 1
#define VIVY_UNIX  1
#elif defined(Q_OS_UNIX)
#define VIVY_LINUX 0
#define VIVY_UNIX  1
#else
#define VIVY_LINUX 0
#define VIVY_UNIX  0
#endif

namespace Vivy::Utils
{
class HostOsInfo {
public:
    static constexpr OsType hostOs()
    {
        if constexpr (VIVY_WIN32)
            return OsTypeWindows;
        else if constexpr (VIVY_LINUX)
            return OsTypeLinux;
        else if constexpr (VIVY_MACOS)
            return OsTypeMac;
        else if constexpr (VIVY_UNIX)
            return OsTypeOtherUnix;
        else
            return OsTypeOther;
    }

    enum HostArchitecture {
        HostArchitectureX86,
        HostArchitectureAMD64,
        HostArchitectureItanium,
        HostArchitectureArm,
        HostArchitectureUnknown
    };
    static HostArchitecture hostArchitecture();

    static constexpr bool isWindowsHost() { return hostOs() == OsTypeWindows; }
    static constexpr bool isLinuxHost() { return hostOs() == OsTypeLinux; }
    static constexpr bool isMacHost() { return hostOs() == OsTypeMac; }
    static constexpr bool isAnyUnixHost() { return VIVY_UNIX; }

    static QString withExecutableSuffix(const QString &executable)
    {
        return OsSpecificAspects::withExecutableSuffix(hostOs(), executable);
    }

    static void setOverrideFileNameCaseSensitivity(Qt::CaseSensitivity sensitivity);
    static void unsetOverrideFileNameCaseSensitivity();

    static Qt::CaseSensitivity fileNameCaseSensitivity()
    {
        return m_useOverrideFileNameCaseSensitivity
                   ? m_overrideFileNameCaseSensitivity
                   : OsSpecificAspects::fileNameCaseSensitivity(hostOs());
    }

    static QChar pathListSeparator() { return OsSpecificAspects::pathListSeparator(hostOs()); }

    static Qt::KeyboardModifier controlModifier()
    {
        return OsSpecificAspects::controlModifier(hostOs());
    }

    static bool canCreateOpenGLContext(QString *errorMessage);

private:
    static Qt::CaseSensitivity m_overrideFileNameCaseSensitivity;
    static bool m_useOverrideFileNameCaseSensitivity;
};

} // namespace Utils
