#pragma once

#include "Lib/CRTPStore.hh"
#include "Lib/Document/VivyDocument.hh"
#include "Lib/Script/ScriptDocument.hh"

namespace Vivy::Script
{
class ModuleDeclaration;
}

namespace Vivy
{
class ScriptStore final : public CRTPStore<ScriptStore, ScriptDocument> {
    VIVY_STORAGE_CLASS(ScriptStore, ScriptDocument)

    static constexpr inline int outBufferSize = 1024;

public:
    using Item = std::tuple<Uuid, QString>;

    explicit ScriptStore() noexcept;

    // Load scripts
    void resetLoadedScripts() noexcept;
    void loadScriptFolder(const QString &folderPath);
    std::vector<Item> getLoadedScripts() const noexcept;
    bool executeScript(Uuid) noexcept;
    bool executeScript(Uuid, std::shared_ptr<Vivy::VivyDocument>) noexcept;

    // Get modules from scripts
    const std::vector<std::string_view> getLoadedModules() const noexcept;
    const Script::ModuleDeclaration *getModule(const std::string_view) const noexcept;
};
}
