/// This file was hand-written to wrap the generated C ABI into a C++ wrapper.

#include "VVLib.h"

#include <algorithm>
#include <string>
#include <string_view>

namespace VVLib
{
class ASSContainer;
class ASSLine;
class ASSSyllabe;

/// ASSSyllabe modelize each syllabe of each line in the ASS file.
class ASSSyllabe {
    C::ASSSyllabe *const self;
    friend ASSLine;

    ASSSyllabe(C::ASSSyllabe *const zelf) noexcept
        : self(zelf)
    {
    }

public:
    const std::string_view content() const noexcept
    {
        const C::StringSlice slice = C::ASSSyllabeGetContent(self);
        return std::string_view(slice.str, slice.len);
    }
    char operator[](int i) noexcept { return content()[static_cast<size_t>(i)]; }
    char operator[](int i) const noexcept { return content()[static_cast<size_t>(i)]; }

    void set_duration(int dur) noexcept { C::ASSSyllabeSetDuration(self, dur); }
    void set_start(int start) noexcept { C::ASSSyllabeSetStartTime(self, start); }
    void set_fini(int fini) noexcept { C::ASSSyllabeSetFiniTime(self, fini); }
    int duration() const noexcept { return C::ASSSyllabeGetDuration(self); }
    int start() const noexcept { return C::ASSSyllabeGetStartTime(self); }
    int fini() const noexcept { return C::ASSSyllabeGetFiniTime(self); }
};

/// ASSLine modelize each line of an ASS file.
class ASSLine {
    C::ASSLine *const self;
    friend ASSContainer;

    ASSLine(C::ASSLine *const zelf) noexcept
        : self(zelf)
    {
    }

public:
    int syllabes_count() const noexcept { return C::ASSLineGetSyllabesCount(self); }
    ASSSyllabe operator[](int i) noexcept { return syllabe_at(i); }
    const ASSSyllabe operator[](int i) const noexcept { return syllabe_at(i); }
    const ASSSyllabe syllabe_at(int i) const noexcept
    {
        return const_cast<ASSLine *>(this)->syllabe_at(i);
    }
    ASSSyllabe syllabe_at(int i) noexcept
    {
        return ASSSyllabe(C::ASSLineGetSyllabeAt(self, std::clamp(i, 0, this->syllabes_count())));
    }

    bool is_commented() const noexcept { return C::ASSLineIsCommented(self); }
    const std::string_view style() const
    {
        const C::StringSlice slice = C::ASSLineGetStyle(self);
        return std::string_view(slice.str, slice.len);
    }

    void set_start(int start) noexcept { C::ASSLineSetStartTime(self, start); }
    void set_fini(int fini) noexcept { C::ASSLineSetFiniTime(self, fini); }
    void set_duration(int dur) noexcept { C::ASSLineSetDuration(self, dur); }
    int start() const noexcept { return C::ASSLineGetStartTime(self); }
    int fini() const noexcept { return C::ASSLineGetFiniTime(self); }
    int duration() const noexcept { return C::ASSLineGetDuration(self); }
};

/// The container wraps everything around an ASS file.
class ASSContainer {
    C::ASSContainer *const self;

    ASSContainer(const char *path) noexcept
        : self(C::ASSContainerFromFile(const_cast<char *>(path)))
    {
    }

public:
    ~ASSContainer() noexcept { C::ASSContainerDrop(self); }

    static ASSContainer from_file(const char *path) noexcept { return ASSContainer(path); }
    static ASSContainer from_file(std::string path) noexcept { return from_file(path.c_str()); }
    static ASSContainer from_file(std::string_view path) noexcept
    {
        return from_file(std::string(path));
    }

    int styles_count() const noexcept { return C::ASSContainerGetStylesCount(self); }
    C::ASSStyle *style(std::string_view name) const noexcept
    {
        C::StringSlice style{ .len = name.length(), .str = name.data() };
        return C::ASSContainerGetStyleByName(self, style);
    }
    std::string_view style(int idx) const noexcept
    {
        const C::StringSlice slice = C::ASSContainerGetStyleNameAt(self, idx);
        return std::string_view(slice.str, slice.len);
    }

    int lines_count() const noexcept { return C::ASSContainerGetLinesCount(self); }
    ASSLine operator[](int i) noexcept { return line_at(i); }
    const ASSLine operator[](int i) const noexcept { return line_at(i); }
    const ASSLine line_at(int i) const noexcept
    {
        return const_cast<ASSContainer *>(this)->line_at(i);
    }
    ASSLine line_at(int i) noexcept
    {
        return ASSLine(C::ASSContainerGetLineAt(self, std::clamp(i, 0, this->lines_count())));
    }
};
}
