use crate::{lowerer::*, ValueContext};
use vvs_lang::ast::*;
use vvs_llvm::*;

pub(super) struct FunctionLowerer<'a> {
    llvm: LLVMLowerContext,

    /// The global context, we store a reference, then we will add it the arguments of the
    /// function.
    ctx: &'a ValueContext,

    /// The llvm function.
    value: LLVMValueRef,

    /// The vivy function.
    function: &'a ASTFunction,
}

impl<'a> FunctionLowerer<'a> {
    pub fn new(llvm: LLVMLowerContext, ctx: &'a ValueContext, v: LLVMValueRef, f: &'a ASTFunction) -> Self {
        Self { llvm, ctx, value: v, function: f }
    }

    pub fn lower(self) {
        let Self { llvm: llvm @ (_, c, b), value, function, .. } = self;
        let ASTFunction { arguments, content, returns, .. } = function;
        let mut ctx = self.ctx.clone();

        let mut ctx = unsafe {
            LLVMPositionBuilderAtEnd(b, LLVMAppendBasicBlockInContext(c, value, c"entry".as_ptr()));
            for (idx, arg) in arguments.iter().enumerate() {
                let ty = arg
                    .get_specified_type()
                    .expect("the type should be specified at this point");
                let llvm_ty = TypeLowerer::new(c, ty).lower();
                let arg_name = CString::new(arg.name().as_ref()).expect("invalid name");
                let arg_param = LLVMGetParam(value, idx.try_into().expect("too many arguments"));
                let ptr = LLVMBuildAlloca(b, llvm_ty, arg_name.as_ptr());
                LLVMBuildStore(b, arg_param, ptr);
                ctx.register(arg.name(), Value::Alloca(ty.clone(), llvm_ty, ptr));
            }

            InstructionLowerer::new(llvm, value, content, returns.clone(), ctx).lower()
        };

        let to_drop = ctx.get_values_to_drop();
        unsafe { LLVMFunctionIter::new(value) }
            .filter(|&bb| unsafe {
                LLVMBasicBlockIter::new(bb)
                    .last()
                    .map_or(false, |instr| LLVMGetInstructionOpcode(instr) == LLVMOpcode::LLVMRet)
            })
            .for_each(|bb| unsafe { DropLowerer::new(c, value, &to_drop, bb, &ctx) }.lower());
    }
}
