use vvs_lang::ast::*;
use vvs_llvm::*;
use vvs_runtime_types::{types::*, vvll::LLVMExported};

pub(super) struct TypeLowerer<'a> {
    c: LLVMContextRef,
    vivy_type: &'a ASTType,
}

impl<'a> TypeLowerer<'a> {
    pub(super) fn new(c: LLVMContextRef, vivy_type: &'a ASTType) -> Self {
        Self { c, vivy_type }
    }

    fn sub_lowerer(&self, ty: &'a ASTType) -> Self {
        Self { c: self.c, vivy_type: ty }
    }

    pub(super) fn lower(self) -> LLVMTypeRef {
        let Self { c, vivy_type } = self;
        match vivy_type {
            // Easy stuff...
            ASTType::Nil => unsafe { LLVMVoidTypeInContext(c) },
            ASTType::Integer => unsafe { LLVMInt32TypeInContext(c) },
            ASTType::Floating => unsafe { LLVMFloatTypeInContext(c) },
            ASTType::Boolean => unsafe { LLVMInt1TypeInContext(c) },

            // Functions
            ASTType::Function(args, returns) => unsafe {
                let mut args = args.iter().map(|ty| self.sub_lowerer(ty).lower()).collect::<Vec<_>>();
                LLVMFunctionType(
                    self.sub_lowerer(returns).lower(),
                    args.as_mut_ptr(),
                    args.len().try_into().expect("too many arguments"),
                    0,
                )
            },

            // The tuple
            ASTType::Tuple(inner) => unsafe {
                let mut inner = inner.iter().map(|ty| self.sub_lowerer(ty).lower()).collect::<Vec<_>>();
                LLVMStructType(inner.as_mut_ptr(), inner.len().try_into().expect("too many elements"), 0)
            },

            // Special structs, query the runtime for that...
            ASTType::Any => unsafe { VVRTAny::llvm_type(c) },
            ASTType::String => unsafe { VVRTString::llvm_type(c) },
            ASTType::Syllabe => unsafe { VVRTSyllabe::llvm_type(c) },
            ASTType::Line => unsafe { VVRTLine::llvm_type(c) },
            ty if ty.is_table() => unsafe { VVRTTable::llvm_type(c) },
            ty if ty.is_variant() => unsafe { VVRTVariant::llvm_type(c) },
            ty if ty.is_sequence() => unsafe { VVRTSeq::llvm_type(c) },

            _ => unreachable!(),
        }
    }
}
