use crate::ast::*;

/// A function. Can also represent a job.
#[derive(Debug, Clone)]
pub struct ASTFunction {
    pub name: ASTString,
    pub returns: ASTType,
    pub arguments: Vec<ASTVar>,
    pub content: ASTInstr,

    pub span: ASTSpan,
}

/// The signature of a callable thing in Vivy Script.
pub type ASTFunctionSignature = (Vec<ASTType>, ASTType);

impl ASTFunction {
    /// Get the signature of a function.
    #[inline]
    pub fn signature(&self) -> ASTFunctionSignature {
        let args = self
            .arguments
            .iter()
            .flat_map(ASTVar::get_specified_type)
            .cloned()
            .collect();
        (args, self.returns.clone())
    }

    /// Get the type of this function, as a function tpe.
    #[inline]
    pub fn function_type(&self) -> ASTType {
        let (args, returns) = self.signature();
        ASTType::Function(args, Box::new(returns))
    }
}
