use crate::ast::*;
use std::ops::Deref;

/// Trait for objects that are typed and may be values known at compile time.
///
/// As long a a type can be dereferenced into a [MaybeConstExpr] thing, it is [MaybeConstExpr].
pub trait MaybeConstExpr: Typed {
    /// Tells whether the thing is a constant expression or not.
    fn is_const_expr(&self) -> bool;

    /// Evaluate the expression, if it is indeed a constant expression then returns the result,
    /// otherwise return [None].
    fn eval_const_expr(&self) -> Option<ASTConst>;
}

impl<S: MaybeConstExpr, T: Deref<Target = S>> MaybeConstExpr for T {
    #[inline]
    fn is_const_expr(&self) -> bool {
        self.deref().is_const_expr()
    }

    #[inline]
    fn eval_const_expr(&self) -> Option<ASTConst> {
        self.deref().eval_const_expr()
    }
}
