//! In this file we declare things needed to do the pattern matching or boolean check for
//! conditionals.
//!
//! Note that we can only pattern match variants.

use crate::ast::*;

/// A condition, two expression must be equal, or should we try to match a thing.
#[derive(Debug, PartialEq, Clone)]
pub enum ASTCondition {
    /// Usual if check.
    BooleanTrue(ASTExpr),

    /// Does the expression can be pattern-matched or not. This is only for variants.
    BindVariant { as_type: ASTType, variant: ASTString, pattern: Vec<ASTPatternElement>, source: ASTVar },

    /// Tells whever a variable is coercible into a type or not.
    CoercibleInto { new: ASTVar, as_type: ASTType, source: ASTVar },
}

/// When we match a pattern, we can do conditionals on the value of elements, or we can bind
/// variables that will be visible in the if/elseif/if-let statement.
#[derive(Debug, PartialEq, Clone)]
pub enum ASTPatternElement {
    /// Must be equal to this thing.
    Const(ASTConst),

    /// Expose this field of the struct in the block if the struct matched.
    Var(ASTVar),
}

impl ASTPatternElement {
    pub fn span(&self) -> Option<ASTSpan> {
        match self {
            ASTPatternElement::Const(_) => None,
            ASTPatternElement::Var(var) => Some(var.span()),
        }
    }
}
