use super::ASTString;
use hashbrown::HashMap;
use std::{cell::RefCell, collections::hash_map::DefaultHasher, hash::Hasher, rc::Rc};

/// Used when iterating into the module with mut access, we might need to access the string
/// cache...
#[derive(Debug, Clone, Default)]
pub struct ASTStringCacheHandle {
    strings: Rc<RefCell<HashMap<u64, ASTString>>>,
}

impl ASTStringCacheHandle {
    /// Get the id of a string.
    fn get_id(str: impl AsRef<str>) -> u64 {
        let mut hasher = DefaultHasher::new();
        hasher.write(str.as_ref().as_bytes());
        hasher.finish()
    }

    /// Get or create a string in the cache.
    pub fn get(&self, str: impl AsRef<str>) -> ASTString {
        self.strings
            .borrow_mut()
            .entry(Self::get_id(str.as_ref()))
            .or_insert_with(|| ASTString::from(str.as_ref()))
            .clone()
    }
}
