use crate::ast::*;
use std::ops::Deref;

/// Trait for objects that are types.
///
/// As long a a type can be dereferenced into a [Typed] thing, it is [Typed].
pub trait Typed {
    /// Get the type of the object.
    fn get_type(&self, ctx: &ASTTypeContext) -> ASTType;
}

impl<S: Typed, T: Deref<Target = S>> Typed for T {
    #[inline]
    fn get_type(&self, ctx: &ASTTypeContext) -> ASTType {
        self.deref().get_type(ctx)
    }
}
