//! Here we define the things that will be exported to C/C++ code. We use PascalCase as it is not
//! very common and getters must be of the form `{StructName}{Get,Set}{TheField}` and methods of
//! the form `{StructName}{FunctionName}`. For free functions we use `{TheCrate}{FunctionName}`.

#![allow(non_snake_case)]
#![no_std]

extern crate alloc;

pub mod ass_elements;
pub mod ass_style;
mod aux_table;
pub mod vivy;

use crate::aux_table::AuxTable;
use core::{ffi::c_char, marker, slice, str};

/// Represents a string slice, the user may not modify this slice, never! Note that the string is
/// not null terminated and may contains null bytes in it, use the len attribute to get the length
/// of this string and convert it to your linking.
///
/// # Safety
/// Note that you must always put unicode inside a string slice!
#[repr(C)]
pub struct StringSlice<'a> {
    len: usize,
    str: *const c_char,
    _data: marker::PhantomData<&'a c_char>,
}

impl<'a> StringSlice<'a> {
    /// # Safety
    /// We expect the content of the string slice to be valid UTF6...
    #[inline]
    pub unsafe fn as_str(&self) -> &'a str {
        str::from_utf8_unchecked(slice::from_raw_parts(self.str as *const _, self.len))
    }
}

impl<'a, S> From<S> for StringSlice<'a>
where
    S: AsRef<str>,
{
    #[inline]
    fn from(value: S) -> Self {
        let value = value.as_ref();
        StringSlice { len: value.len(), str: value.as_ptr() as *const _, _data: marker::PhantomData }
    }
}
