use crate::prelude::*;
use llvm_sys::{core::*, prelude::*};

crate::bindings::declare! { mut LLVMBasicBlockRef as BB<'a> }

impl<'a> BB<'a> {
    /// Get the insturction count of this basic block.
    pub fn len(&self) -> usize {
        let len = unsafe { LLVMGetLastInstruction(self.as_ptr()).offset_from(LLVMGetFirstInstruction(self.as_ptr())) };
        assert!(len >= 0, "something went really wrong");
        len as usize
    }

    /// Tells wether there are any instructions in this basic block.
    pub fn is_empty(&self) -> bool {
        self.len() == 0
    }

    /// Iterate over the instructions of a basic block.
    pub fn iter(&'a self) -> BBIter<'a, &'a BB<'a>> {
        match self.is_empty() {
            true => BBIter { curr: None, last: None, bb: self },
            false => todo!(),
        }
    }

    /// Iterate over the instructions of a basic block in a mutable way.
    pub fn iter_mut(&'a mut self) -> BBIter<'a, &'a mut BB<'a>> {
        match self.is_empty() {
            true => BBIter { curr: None, last: None, bb: self },
            false => todo!(),
        }
    }
}

impl Eq for BB<'_> {}
impl PartialEq for BB<'_> {
    fn eq(&self, other: &Self) -> bool {
        self.inner == other.inner
    }
}

mod sealed {
    pub trait BBRef<'a>: AsRef<super::BB<'a>> {}
}

impl<'a> sealed::BBRef<'a> for &'a BB<'a> {}
impl<'a> sealed::BBRef<'a> for &'a mut BB<'a> {}

impl<'a> AsRef<BB<'a>> for &'a BB<'a> {
    fn as_ref(&self) -> &BB<'a> {
        self
    }
}

impl<'a> AsRef<BB<'a>> for &'a mut BB<'a> {
    fn as_ref(&self) -> &BB<'a> {
        self
    }
}

/// Iterate over the instructions in a basic block.
pub struct BBIter<'a, B: sealed::BBRef<'a>> {
    curr: Option<Value<'a>>,
    last: Option<Value<'a>>,
    bb: B,
}

impl<'a, B: sealed::BBRef<'a>> Iterator for BBIter<'a, B> {
    type Item = Value<'a>;

    fn next(&mut self) -> Option<Self::Item> {
        match (self.curr.take(), self.last.as_ref()) {
            (Some(_), None) | (None, Some(_)) => unreachable!(),
            (None, None) => None,
            (Some(curr), Some(last)) if curr == *last => {
                self.last = None;
                Some(curr)
            }
            (Some(curr), Some(_)) => unsafe {
                self.curr = Some(Value::from_ptr(LLVMGetNextInstruction(curr.as_ptr())));
                Some(curr)
            },
        }
    }

    fn last(self) -> Option<Self::Item> {
        self.last
    }

    fn count(self) -> usize {
        self.bb.as_ref().len()
    }

    fn size_hint(&self) -> (usize, Option<usize>) {
        let size = self.bb.as_ref().len();
        (size, Some(size))
    }
}
