use crate::prelude::*;
use llvm_sys::{core::*, prelude::*};

crate::bindings::declare! { mut LLVMBuilderRef as Builder<'a> }

macro_rules! build {
    ($name:ident ($b:ident $(, $arg:ident : $ty:ty)*$(,)?) $doc:literal $expr:expr) => {
        paste::paste! {
            #[doc = concat!($doc,
                "\nUsage: `b.build_", stringify!($name), "(" $(, concat!(stringify!($arg), ","))*, ")`"
            )]
            pub fn [< build_$name >](&mut self $(, $arg: $ty)*) -> Value {
                let $b = self.as_ptr();
                unsafe { Value::from_ptr($expr) }
            }
        }
    };
}

impl<'a> Builder<'a> {
    /// Position the builder at the end of a basic block.
    pub fn position_at_end(&mut self, bb: &BB) -> &mut Self {
        unsafe { LLVMPositionBuilderAtEnd(self.as_ptr(), bb.as_ptr()) };
        self
    }

    /// Position the builder before a value
    pub fn position_before(&mut self, value: &Value) -> &mut Self {
        unsafe { LLVMPositionBuilderBefore(self.as_ptr(), value.as_ptr()) };
        self
    }

    build! { cond_jump(b, cond: Value, then_bb: &BB, else_bb: &BB) "Build a conditional jump to another basic block" {
        LLVMBuildCondBr(b, cond.as_ptr(), then_bb.as_ptr(), else_bb.as_ptr())
    }}
    build! { jump(b, bb: &BB) "Build a jump to another basic block" LLVMBuildBr(b, bb.as_ptr()) }
    build! { ret_void(b)      "Build a `return void` instruction"   LLVMBuildRetVoid(b) }
}

impl Drop for Builder<'_> {
    fn drop(&mut self) {
        unsafe { LLVMDisposeBuilder(self.inner) }
    }
}
