use crate::prelude::*;
use llvm_sys::{core::*, prelude::*, LLVMLinkage, LLVMVisibility};
use std::ffi::{CString, NulError};

crate::bindings::declare! { mut LLVMModuleRef as Module<'a> }

impl Module<'_> {
    /// Add a new function in this module.
    ///
    /// Note that by default all functions are private.
    pub fn add_function(&mut self, name: impl AsRef<str>, ty: FunctionType) -> Result<FunctionDeclaration, NulError> {
        let function = unsafe { LLVMAddFunction(self.inner, CString::new(name.as_ref())?.as_ptr(), ty.as_ptr()) };

        // All the functions are private to the module!
        unsafe {
            LLVMSetLinkage(function, LLVMLinkage::LLVMLinkerPrivateLinkage);
            LLVMSetVisibility(function, LLVMVisibility::LLVMHiddenVisibility);
        }

        Ok(unsafe { FunctionDeclaration::from_ptr(function) })
    }
}

impl Drop for Module<'_> {
    fn drop(&mut self) {
        unsafe { LLVMDisposeModule(self.inner) }
    }
}
