#![warn(missing_docs)]
#![allow(clippy::large_enum_variant)]

//! # Vivy Script Parser.
//!
//! `vvs_parser` is a lossless parser for Vivy Script, inspired by Lua and Luau.

mod ast;
mod error;
mod node;
mod private;
mod short_string;
mod tokenizer;
mod traits;
mod util;
mod visitors;
mod vivy;

#[cfg(test)]
mod tests;

use crate::{error::Error, short_string::ShortString};

/// Crates depending upon [vss_parser] may import the prelude.
pub mod prelude {
    /// Re-export everything that is linked to the [Ast].
    pub mod ast {
        pub use crate::{ast::*, node::*, short_string::ShortString, tokenizer::*, visitors::*};
    }

    /// Parsers, get the raw representation of things without transforming or checking the
    /// resulting things. May be used for debug or test only.
    pub mod parser {
        /// Creates an [`Ast`](ast::Ast) for VivyScript code.
        ///
        /// Note that the resulting code may not be entirely valid! You need to pass it into the
        /// [CompilerPipeline] for that!
        ///
        /// # Errors
        /// If the code passed cannot be tokenized, a TokenizerError will be returned.
        /// If the code passed is not valid Lua 5.1 code, an AstError will be returned,
        /// specifically AstError::UnexpectedToken.
        pub fn parse_lua_tree(code: &str) -> Result<crate::ast::Ast, Vec<crate::Error>> {
            crate::ast::AstResult::parse_fallible(code).into_result()
        }

        /// Given code, will produce an [`ast::OptionTableResult`]. This OptionTableResult always produces
        /// some [OptionTableResult], regardless of errors. If a partial Ast is produced (i.e. if there are
        /// any errors), a few guarantees are lost. The same remarks can be made as [parse_fallible].
        pub fn parse_options_tree(code: &str) -> Result<crate::ast::OptionTable, Vec<crate::Error>> {
            crate::ast::OptionTableResult::parse_fallible(code).into_result()
        }
    }

    pub use crate::error::Error as VVSParserError;
    pub use crate::vivy::*;
}
