//! Vivy specific logic

mod error_report;
mod frontend_pipeline;
mod library;
mod main_program;
mod passes;
mod search_path;
mod symbol_table;

use std::{borrow::Cow, fs, io, path::Path};

pub use self::{error_report::ErrorReport, frontend_pipeline::*, main_program::*, passes::UserFrontendPass};

/// The representation of a source file.
pub struct SourceCode {
    name: Cow<'static, str>,
    code: Cow<'static, str>,
}

impl SourceCode {
    /// Create a new source file from a file.
    pub fn from_path(path: impl AsRef<Path>) -> io::Result<Self> {
        Ok(Self {
            name: Cow::Owned(
                path.as_ref()
                    .file_name()
                    .ok_or_else(|| io::Error::new(io::ErrorKind::Other, "path doesn't have a file name"))?
                    .to_string_lossy()
                    .to_string(),
            ),
            code: Cow::Owned(fs::read_to_string(path.as_ref())?),
        })
    }

    /// Create a new source file from memory.
    pub fn from_memory(code: Cow<'static, str>) -> Self {
        Self { name: Cow::Borrowed("--"), code }
    }

    /// Get the name of the source file.
    pub fn name(&self) -> &str {
        &self.name
    }
}
