use crate::{ast::*, node::Node, tokenizer::*, vivy::passes::declare_pass, ShortString};

declare_pass! {
    /// The option optimization is here to substitute the options with the values chosen by the
    /// user.
    #[allow(dead_code)]
    transform::OptsTransform: 'a {}

    aux    { options: &'a OptionTable                 }
    result { unused:  Vec<(ShortString, ShortString)> }
}

impl crate::visitors::VisitorMut for OptsTransform<'_> {
    fn visit_expression_end(&mut self, expression: Expression) -> Expression {
        match expression {
            Expression::Var(Var::Expression(var)) => self.process_var_expression(var),
            expression => expression,
        }
    }
}

impl OptsTransform<'_> {
    fn process_var_expression(&mut self, var: Box<VarExpression>) -> Expression {
        let module = match var.prefix() {
            Prefix::Name(TokenReference {
                token: Token { token_type: TokenType::Identifier { identifier }, .. },
                ..
            }) if self.options.has_section(identifier) => identifier.clone(),
            _ => return Expression::Var(Var::Expression(var)),
        };

        match var.suffixes().next() {
            Some(Suffix::Index(Index::Dot {
                name: TokenReference { token: Token { token_type: TokenType::Identifier { ref identifier }, .. }, .. },
                ..
            })) if var.suffixes().count() == 1 => self.options.get(&module, identifier).cloned().unwrap_or_else(|| {
                self.error(var.tokens().next().unwrap(), format!("failed to find option `{module}.{identifier}`"));
                Expression::Symbol(TokenReference::basic_symbol("nil"))
            }),

            Some(Suffix::Index(name)) => {
                self.error(name.tokens().next().unwrap(), "expected a reference to an option, e.g. `module.option`");
                Expression::Symbol(TokenReference::basic_symbol("nil"))
            }

            _ => Expression::Var(Var::Expression(var.clone())),
        }
    }
}
