//! Options that can be passed to some functions to control the behaviour.

/// Control the behaviour of the [XDGFolder::find] function on conflicts.
#[derive(Debug, Default, Clone, Copy, PartialEq, Eq)]
pub enum XDGFindBehaviour {
    /// Returns only the files that already exists. If no file is found, returns the file with the
    /// post priority that can be created, usually where the user want this file to exists. If you
    /// don't want to merge config/data files/folders this can be a sane default.
    #[default]
    FirstOrCreate,

    /// Returns only the files that already exists. If you want to merge config/data files/folders
    /// this can be a sane default.
    ExistingOnly,

    /// Only returns files that don't already exists and may be created. Note that even if a file
    /// can be created by the user, it won't necessarily means that you will be able to, keep in
    /// mind that race conditions may occur here.
    NonExistingOnly,

    /// Returns all files, the ones that exist and the ones that may be created. The user will need
    /// to handle the returned files as he sees fit.
    AllFiles,
}
