#include "PreCompiledHeaders.hh"
#include "UI/AbstractDocumentView.hh"

using namespace Vivy;

AbstractDocumentView::AbstractDocumentView(AbstractDocumentView::Type type,
                                           QWidget *parent) noexcept
    : QMainWindow(parent)
    , documentType(type)
{
    setStatusBar(nullptr);
    setMenuBar(nullptr);
    setTabPosition(Qt::TopDockWidgetArea, QTabWidget::North);
    setTabPosition(Qt::BottomDockWidgetArea, QTabWidget::South);
    setTabPosition(Qt::LeftDockWidgetArea, QTabWidget::West);
    setTabPosition(Qt::RightDockWidgetArea, QTabWidget::East);
}

void
AbstractDocumentView::allowToCloseAllDocks() noexcept
{
    // Allow to close all dock widgets
    for (UnclosableDockWidget *const dock : dockWidgets)
        dock->allowClose();
}

// A utility function to delete all the child widgets
void
AbstractDocumentView::deleteAllContent() noexcept
{
    // Delete all widgets
    if (layout() != nullptr) {
        QLayoutItem *item;
        while ((item = layout()->takeAt(0)) != nullptr) {
            delete item->widget();
            delete item;
        }
        delete layout();
    }
}

// Returns actions that must be placed in the view actions to show/hide some of
// the document's widgets
const QList<QAction *> &
AbstractDocumentView::getViewsActions() const noexcept
{
    return viewsActions;
}

AbstractDocumentView::Type
AbstractDocumentView::getType() const noexcept
{
    return documentType;
}

void
AbstractDocumentView::delDockWidget(UnclosableDockWidget **dock) noexcept
{
    // Remove the toggle view action
    QAction *act = (*dock)->toggleViewAction();
    if (viewsActions.contains(act))
        viewsActions.removeAll(act);

    // Remove the widget
    (*dock)->allowClose();
    removeDockWidget(*dock);
    delete *dock;
    *dock = nullptr;

    // Emit the signal
    emit viewActionsChanged();
}
void
AbstractDocumentView::addDockWidget(Qt::DockWidgetArea area, UnclosableDockWidget *dock,
                                    Qt::Orientation orientation) noexcept
{
    // dock->setTitleBarWidget(new QWidget(dock)); // <- to disable the header bar
    QMainWindow::addDockWidget(area, dock, orientation);
    QAction *act = dock->toggleViewAction();
    if (!viewsActions.contains(act))
        viewsActions.prepend(act);
    if (!dockWidgets.contains(dock))
        dockWidgets.append(dock);
    emit viewActionsChanged();
}

bool
AbstractDocumentView::isType(const Utils::DocumentType t) const noexcept
{
    return Utils::toUnderlying(documentType) == Utils::toUnderlying(t);
}
